@@ -3,4 +3,6 @@
 from .target import Target
 from .config import Config
 from .category import Category
+from .ip import IP
 from .organization import Organization
+from .port import Port
